﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ProhlizecObrazku
{
    public partial class frmProchazeniAdresaru : Form
    {
        string currDir;

        int sirkaNahledu = 100;
        int vyskaNahledu = 75;
        int paddingX = 20;
        int paddingY = 20;
        int obrazkuNaRadek;

        AnnotatedPictureBox[] nahledy;
        Label[] popisky;

        public string CurrentDirectory
        {
            get
            {
                return currDir;
            }
            set
            {
                currDir = value;
                zaktualizujNahledy();
            }
        }

        public frmProchazeniAdresaru(string initDir)
        {
            InitializeComponent();

            CurrentDirectory = initDir;

            TreeNode root = directoryView.Nodes.Add(Path.GetPathRoot(initDir));     
            addSubdirectoriesToTreeView(root);
        }

        private void addSubdirectoriesToTreeView(TreeNode parent)
        {
            try
            {
                foreach (string subdir in Directory.GetDirectories(parent.Text))
                    parent.Nodes.Add(subdir);
            }
            catch (UnauthorizedAccessException)
            {
            }
        }

        private void zaktualizujNahledy()
        {
            var obrazky = Directory.GetFiles(currDir, "*.jpg")
                    .Concat(Directory.GetFiles(currDir, "*.jpeg"))
                    .OrderBy(x => x);

            nahledy = new AnnotatedPictureBox[obrazky.Count()];
            popisky = new Label[obrazky.Count()];

            int index = 0;

            obrazkuNaRadek = pnlNahledy.Width / (sirkaNahledu + paddingX);

            if (obrazkuNaRadek == 0)
                obrazkuNaRadek = 1;

            foreach (string obrazek in obrazky)
            {
                AnnotatedPictureBox pic = new AnnotatedPictureBox(obrazek);
                pic.Image = Image.FromFile(obrazek);
                pic.SizeMode = PictureBoxSizeMode.StretchImage;
                pic.Size = new Size(sirkaNahledu, vyskaNahledu);
                pic.Location = new Point(paddingX + (sirkaNahledu + paddingX) * (index % obrazkuNaRadek),
                                         paddingY + (vyskaNahledu + paddingY) * (index / obrazkuNaRadek));
                pic.Click += new EventHandler(pic_Click);
                pic.DoubleClick += new EventHandler(pic_DoubleClick);

                Label label = new Label();
                label.Text = Path.GetFileName(obrazek);
                label.TextAlign = ContentAlignment.TopCenter;
                label.Width = pic.Width;
                label.Location = new Point(pic.Location.X, pic.Location.Y + pic.Height);

                nahledy[index] = pic;
                popisky[index] = label;
                index++;
            }

            pnlNahledy.Controls.Clear();

            pnlNahledy.Controls.AddRange(nahledy);
            pnlNahledy.Controls.AddRange(popisky);
        }

        void pic_Click(object sender, EventArgs e)
        {
            propertyGrid.SelectedObject = (sender as AnnotatedPictureBox).Image;
        }

        void pic_DoubleClick(object sender, EventArgs e)
        {
            AnnotatedPictureBox pic = sender as AnnotatedPictureBox;
            string obrazek = pic.ImagePath;

            frmZobrazeniObrazku zobrazeni = new frmZobrazeniObrazku(obrazek);
            zobrazeni.Show(this);
        }

        private void frmProchazeniAdresaru_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
                this.Close();
        }

        private void directoryView_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (e.Node.Nodes.Count == 0)
                addSubdirectoriesToTreeView(e.Node);

            CurrentDirectory = e.Node.Text;
        }
    }
}
