﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ProhlizecObrazku
{
    public partial class frmZobrazeniObrazku : Form
    {
        private enum Rotation
        {
            None = 0,
            Deg90 = 1,
            Deg180 = 2,
            Deg270 = 3
        }

        private string currPic;
        private int vyska, sirka;
        private Image obrazek;
        private Bitmap grayscale;
        private Rotation rotation = Rotation.None;
        private Dictionary<Rotation, Point[]> destPointDictionary = new Dictionary<Rotation, Point[]>();
        private bool horizontalFlip = false, verticalFlip = false, useGrayscale = false;
        private float initialScale = 1.0f;
        private float totalZoom = 1.0f;
        private float zoomSpeed = 0.9f;
        private string[] obrazkyVeSlozce;
        private int index;
            

        public frmZobrazeniObrazku(string initPic)
        {
            InitializeComponent();

            this.CurrentPicture = initPic;

            destPointDictionary.Add(Rotation.None, new Point[] { new Point(0, 0),
                                                                 new Point(sirka, 0),
                                                                 new Point(0, vyska) });

            destPointDictionary.Add(Rotation.Deg90, new Point[] { new Point(sirka, 0),
                                                                  new Point(sirka, vyska),
                                                                  new Point(0, 0) });

            destPointDictionary.Add(Rotation.Deg180, new Point[] { new Point(sirka, vyska),
                                                                   new Point(0, vyska),
                                                                   new Point(sirka, 0) });

            destPointDictionary.Add(Rotation.Deg270, new Point[] { new Point(0, vyska),
                                                                   new Point(0, 0),
                                                                   new Point(sirka, vyska) });


            string currDir = Path.GetDirectoryName(currPic);

            string[] obrazkyJPGVeSlozce = Directory.GetFiles(currDir, "*.jpg");
            string[] obrazkyJPEGVeSlozce = Directory.GetFiles(currDir, "*.jpeg");
            obrazkyVeSlozce = new string[obrazkyJPEGVeSlozce.Length + obrazkyJPGVeSlozce.Length];

            obrazkyJPGVeSlozce.CopyTo(obrazkyVeSlozce, 0);
            obrazkyJPEGVeSlozce.CopyTo(obrazkyVeSlozce, obrazkyJPGVeSlozce.Length);

            Array.Sort(obrazkyVeSlozce);

            for (int i = 0; i < obrazkyVeSlozce.Length; i++)
                if (obrazkyVeSlozce[i] == currPic)
                {
                    index = i;
                    break;
                }
        }

        public string CurrentPicture
        {
            get
            {
                return currPic;
            }
            set
            {
                currPic = value;
                nactiObrazek();
            }
        }

        private void nactiObrazek()
        {

            grayscale = null;
            obrazek = Image.FromFile(currPic);
            sirka = obrazek.Width;
            vyska = obrazek.Height;
            initialScale = 1.0f;

            int windowWidth = sirka;
            int windowHeight = vyska;

            if (windowWidth > Screen.GetBounds(this).Width)
            {
                float scale = (float)Screen.GetBounds(this).Width / (float)windowWidth;
                windowWidth = (int)(windowWidth * scale);
                windowHeight = (int)(windowHeight * scale);
                initialScale *= scale;
            }

            if (windowHeight > Screen.GetBounds(this).Height)
            {
                float scale = (float)Screen.GetBounds(this).Height / (float)windowHeight;
                windowWidth = (int)(windowWidth * scale);
                windowHeight = (int)(windowHeight * scale);
                initialScale *= scale;
            }

            this.Width = windowWidth;
            this.Height = windowHeight;
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            Graphics graphics = e.Graphics;

            Point[] destPointsFromRotation = destPointDictionary[rotation];
            Point[] destPoints = new Point[3];

            float totalScale = initialScale * totalZoom;

            for (int i = 0; i < 3; i++)
            {
                if (horizontalFlip)
                    destPoints[i].X = sirka - destPointsFromRotation[i].X;
                else
                    destPoints[i].X = destPointsFromRotation[i].X;
                if (verticalFlip)
                    destPoints[i].Y = vyska - destPointsFromRotation[i].Y;
                else
                    destPoints[i].Y = destPointsFromRotation[i].Y;
                destPoints[i].X = (int) (totalScale * destPoints[i].X);
                destPoints[i].Y = (int)(totalScale * destPoints[i].Y);
            }

            Image nakreslit = obrazek;

            if (useGrayscale)
            {
                if (grayscale == null)
                    generateGrayscale();
                nakreslit = grayscale;
            }

            graphics.DrawImage(nakreslit, destPoints);
        }

        private void generateGrayscale()
        {
            Bitmap bitmapa = obrazek as Bitmap;
            grayscale = new Bitmap(sirka, vyska);
            for (int x = 0; x < sirka; x++)
                for (int y = 0; y < vyska; y++)
                {
                    Color barva = bitmapa.GetPixel(x, y);
                    int luminance = (int)(barva.R * 0.3 + barva.G * 0.59+ barva.B*  0.11);
                    grayscale.SetPixel(x, y, Color.FromArgb(luminance, luminance, luminance));
                }
        }

        private void rotateRight()
        {
            rotation = (Rotation)(((int)rotation + 1) % 4);
            this.Invalidate();
        }

        private void rotateLeft()
        {
            int newRotation = (int)rotation - 1;
            if (newRotation < 0)
                newRotation += 4;

            rotation = (Rotation)newRotation;
            this.Invalidate();
        }

        private void frmZobrazeniObrazku_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.L:
                    rotateLeft();
                    break;
                case Keys.R:
                    rotateRight();
                    break;
                case Keys.H:
                    horizontalFlip = !horizontalFlip;
                    this.Invalidate();
                    break;
                case Keys.V:
                    verticalFlip = !verticalFlip;
                    this.Invalidate();
                    break;
                case Keys.G:
                    useGrayscale = !useGrayscale;
                    this.Invalidate();
                    break;
                case Keys.Add:
                    totalZoom /= zoomSpeed;
                    this.Invalidate();
                    break;
                case Keys.Subtract:
                    totalZoom *= zoomSpeed;
                    this.Invalidate();
                    break;
                case Keys.Divide:
                    totalZoom = 1.0f;
                    this.Invalidate();
                    break;
                case Keys.Home:
                    CurrentPicture = obrazkyVeSlozce[0];
                    this.Invalidate();
                    break;
                case Keys.End:
                    CurrentPicture = obrazkyVeSlozce[obrazkyVeSlozce.Length - 1];
                    this.Invalidate();
                    break;
                case Keys.PageDown:
                    index = (index + 1) % obrazkyVeSlozce.Length;
                    CurrentPicture = obrazkyVeSlozce[index];
                    this.Invalidate();
                    break;
                case Keys.PageUp:
                    index--;
                    if (index < 0)
                        index += obrazkyVeSlozce.Length;
                    CurrentPicture = obrazkyVeSlozce[index];
                    this.Invalidate();
                    break;
                case Keys.Enter:
                case Keys.Escape:
                    this.Close();
                    break;
            }
        }

        private void frmZobrazeniObrazku_DoubleClick(object sender, EventArgs e)
        {
            this.Close();
        }

        void frmZobrazeniObrazku_MouseWheel(object sender, MouseEventArgs e)
        {
            float zoomAmount = (float)e.Delta / 120.0f;

            totalZoom *= (float)Math.Pow(zoomSpeed, -zoomAmount);
            this.Invalidate();
        }
    }
}
